// Copyright (C) 2019  Wilko Manger
//
// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this
// file, You can obtain one at http://mozilla.org/MPL/2.0/.

import 'dart:typed_data';
import 'dart:ui';

import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';
import 'package:matrix_sdk/matrix_sdk.dart';
import 'package:url/url.dart';

import 'matrix_cache_manager.dart';

class MatrixImage extends ImageProvider<MatrixImage> {
  /// Matrix URL pointing to the image.
  final Url url;

  final double scale;
  final int width, height;

  final MatrixCacheManager _cacheManager;

  /// A Matrx image. If width and height are provided, downloads a thumbnail.
  MatrixImage(
    this.url, {
    @required Homeserver homeserver,
    this.scale = 1.0,
    this.width,
    this.height,
  }) : _cacheManager = MatrixCacheManager(homeserver);

  Future<Codec> _load(MatrixImage key) async {
    final file = await _cacheManager.getSingleFile(
      key.url.toString(),
      headers: {
        'width': key.width.toString(),
        'height': key.height.toString(),
      },
    );

    if (file == null) {
      return null;
    }

    final bytes = Uint8List.fromList(await file.readAsBytes());

    return PaintingBinding.instance.instantiateImageCodec(bytes);
  }

  @override
  ImageStreamCompleter load(MatrixImage key) {
    return MultiFrameImageStreamCompleter(
      codec: _load(key),
      scale: key.scale,
    );
  }

  @override
  Future<MatrixImage> obtainKey(ImageConfiguration configuration) {
    return SynchronousFuture<MatrixImage>(this);
  }

  @override
  bool operator ==(dynamic other) {
    if (other.runtimeType != runtimeType) return false;

    final MatrixImage typedOther = other;
    return url == typedOther.url && scale == typedOther.scale;
  }

  @override
  int get hashCode => hashValues(url, scale);
}
